@@ -37,6 +37,7 @@ def load(
     filename: str,
     disregard_units: bool = False,
     registration_file_path: Union[str, None] = None,
+    mask: slice = Ellipsis,
 ) -> VelociraptorCatalogue:
     """
     Loads a velociraptor catalogue, producing a VelociraptorCatalogue
@@ -63,6 +64,11 @@ def load(
         additional properties with the catalogue. This is an
         advanced feature. See the documentation for more details.
 
+    mask: Union[None, NDArray[bool], int], optional
+        If a boolean array is provided, it is used to mask all
+        catalogue arrays. If an int is provided, catalogue arrays
+        are masked to the single corresponding element.
+
 
     Returns
     -------
@@ -72,7 +78,9 @@ def load(
         .properties file.
     """
 
-    catalogue = VelociraptorCatalogue(filename, disregard_units=disregard_units)
+    catalogue = VelociraptorCatalogue(
+        filename, disregard_units=disregard_units, mask=mask
+    )
 
     if registration_file_path is not None:
         catalogue.register_derived_quantities(registration_file_path)
