@@ -4,6 +4,7 @@ Main objects for holding information relating to the autoplotter.
 
 from velociraptor import VelociraptorCatalogue
 from velociraptor.autoplotter.lines import VelociraptorLine, valid_line_types
+from velociraptor.autoplotter.box_size_correction import VelociraptorBoxSizeCorrection
 from velociraptor.exceptions import AutoPlotterError
 from velociraptor.observations import load_observations
 
@@ -11,7 +12,7 @@ import velociraptor.autoplotter.plot as plot
 
 from unyt import unyt_quantity, unyt_array, matplotlib_support
 from unyt.exceptions import UnitConversionError
-from numpy import log10, linspace, logspace, array, logical_and
+from numpy import log10, linspace, logspace, array, logical_and, ones
 from matplotlib.pyplot import Axes, Figure, close
 from yaml import safe_load
 from typing import Union, List, Dict, Tuple
@@ -84,8 +85,12 @@ class VelociraptorPlot(object):
     y_bins: unyt_array
     # Select a specific structure type?
     select_structure_type: Union[None, int]
+    exclude_structure_type: Union[None, int]
     structure_mask: Union[None, array]
     selection_mask: Union[None, array]
+    # Apply a box size correction to the plot?
+    correction_directory: str
+    box_size_correction: Union[None, VelociraptorBoxSizeCorrection]
     # Where should the legend and z, a information be placed?
     legend_loc: str
     redshift_loc: str
@@ -94,12 +99,15 @@ class VelociraptorPlot(object):
     observational_data_filenames: List[str]
     observational_data_bracket_width: float
     observational_data_directory: str
+    # global mask
+    global_mask: Union[None, array]
 
     def __init__(
         self,
         filename: str,
         data: Dict[str, Union[Dict, str]],
         observational_data_directory: str,
+        correction_directory: str,
     ):
         """
         Initialise the plot object variables.
@@ -107,6 +115,7 @@ class VelociraptorPlot(object):
         self.filename = filename
         self.data = data
         self.observational_data_directory = observational_data_directory
+        self.correction_directory = correction_directory
 
         self._parse_data()
 
@@ -260,7 +269,7 @@ class VelociraptorPlot(object):
 
         return
 
-    def _parse_structure_type(self) -> None:
+    def _parse_select_structure_type(self) -> None:
         """
         Parses the structure type selector to select_structure_type, as well as
         creating the structure_mask
@@ -276,6 +285,22 @@ class VelociraptorPlot(object):
 
         return
 
+    def _parse_exclude_structure_type(self) -> None:
+        """
+        Parses the structure type excluder to exclude_structure_type, as well as
+        creating the structure_mask
+        """
+        try:
+            self.exclude_structure_type = int(self.data["exclude_structure_type"])
+        except KeyError:
+            self.exclude_structure_type = None
+
+        # Initialise to None; we set/create this when we first have access to the
+        # catalogue in the plotting functions.
+        self.structure_mask = None
+
+        return
+
     def _parse_selection_mask(self) -> None:
         """
         Parses the selection mask selector.
@@ -445,7 +470,8 @@ class VelociraptorPlot(object):
         self._parse_loc()
         self._parse_comment()
         self._parse_lines()
-        self._parse_structure_type()
+        self._parse_select_structure_type()
+        self._parse_exclude_structure_type()
         self._parse_selection_mask()
 
         return
@@ -486,7 +512,8 @@ class VelociraptorPlot(object):
         self._parse_coordinate_histogram_bin("x")
         self._parse_loc()
         self._parse_comment()
-        self._parse_structure_type()
+        self._parse_select_structure_type()
+        self._parse_exclude_structure_type()
         self._parse_selection_mask()
 
         return
@@ -502,6 +529,13 @@ class VelociraptorPlot(object):
 
         self._parse_common_histogramtype()
 
+        try:
+            box_size_correction = str(self.data["box_size_correction"])
+            self.box_size_correction = VelociraptorBoxSizeCorrection(
+                box_size_correction, self.correction_directory
+            )
+        except KeyError:
+            self.box_size_correction = None
         # A bit of a hacky workaround - improve this in the future
         # by combining this functionality properly into the
         # VelociraptorLine methods.
@@ -514,6 +548,7 @@ class VelociraptorPlot(object):
                 start=dict(value=self.x_lim[0].value, units=self.x_lim[0].units),
                 end=dict(value=self.x_lim[1].value, units=self.x_lim[1].units),
             ),
+            box_size_correction=self.box_size_correction,
         )
 
         return
@@ -529,6 +564,13 @@ class VelociraptorPlot(object):
 
         self._parse_common_histogramtype()
 
+        try:
+            box_size_correction = str(self.data["box_size_correction"])
+            self.box_size_correction = VelociraptorBoxSizeCorrection(
+                box_size_correction, self.correction_directory
+            )
+        except KeyError:
+            self.box_size_correction = None
         # A bit of a hacky workaround - improve this in the future
         # by combining this functionality properly into the
         # VelociraptorLine methods.
@@ -542,6 +584,7 @@ class VelociraptorPlot(object):
                 end=dict(value=self.x_lim[1].value, units=self.x_lim[1].units),
                 adaptive=True,
             ),
+            box_size_correction=self.box_size_correction,
         )
 
         return
@@ -736,32 +779,42 @@ class VelociraptorPlot(object):
         name = x.name
 
         if self.structure_mask is not None:
-            x = x[self.structure_mask]
+            # if structure_mask already set, mask and return
+            x_mask = logical_and(self.global_mask, self.structure_mask)
+            x = x[x_mask]
             x.name = name
-        elif self.selection_mask is not None:
+            return x
+
+        # allow all entries by default
+        self.structure_mask = ones(x.shape).astype(bool)
+
+        if self.selection_mask is not None:
             # Create mask
             self.structure_mask = reduce(
                 getattr, self.selection_mask.split("."), catalogue
             ).astype(bool)
-
-            if self.select_structure_type is not None:
-                self.structure_mask = logical_and(
-                    self.structure_mask,
-                    catalogue.structure_type.structuretype
-                    == self.select_structure_type,
+        if self.select_structure_type is not None:
+            if self.select_structure_type == self.exclude_structure_type:
+                raise AutoPlotterError(
+                    f"Cannot simultaneously select and exclude structure"
+                    " type {self.select_structure_type}"
                 )
-
-            x = x[self.structure_mask]
-            x.name = name
-        elif self.select_structure_type is not None:
-            # Need to create mask
-            self.structure_mask = (
-                catalogue.structure_type.structuretype == self.select_structure_type
+            self.structure_mask = logical_and(
+                self.structure_mask,
+                catalogue.structure_type.structuretype == self.select_structure_type,
+            )
+        if self.exclude_structure_type is not None:
+            self.structure_mask = logical_and(
+                self.structure_mask,
+                catalogue.structure_type.structuretype != self.exclude_structure_type,
             )
 
-            x = x[self.structure_mask]
-            x.name = name
+        # combine global and structure masks
+        x_mask = logical_and(self.global_mask, self.structure_mask)
 
+        # apply to the unyt array of values
+        x = x[x_mask]
+        x.name = name
         return x
 
     def _make_plot_scatter(
@@ -1011,13 +1064,18 @@ class AutoPlotter(object):
     plots: List[VelociraptorPlot]
     # Directory containing the observational data.
     observational_data_directory: str
+    # Directory containing box size correction data
+    correction_directory: str
     # Whether or not the plots were created successfully.
     created_successfully: List[bool]
+    # global mask
+    global_mask: Union[None, array]
 
     def __init__(
         self,
         filename: Union[str, List[str]],
         observational_data_directory: Union[None, str] = None,
+        correction_directory: Union[None, str] = None,
     ) -> None:
         """
         Initialises the AutoPlotter object with the yaml filename(s).
@@ -1042,6 +1100,9 @@ class AutoPlotter(object):
             if observational_data_directory is not None
             else ""
         )
+        self.correction_directory = Path(
+            correction_directory if correction_directory is not None else ""
+        )
 
         self.load_yaml()
         self.parse_yaml()
@@ -1072,13 +1133,20 @@ class AutoPlotter(object):
         """
 
         self.plots = [
-            VelociraptorPlot(filename, plot, self.observational_data_directory)
+            VelociraptorPlot(
+                filename,
+                plot,
+                self.observational_data_directory,
+                self.correction_directory,
+            )
             for filename, plot in self.yaml.items()
         ]
 
         return
 
-    def link_catalogue(self, catalogue: VelociraptorCatalogue):
+    def link_catalogue(
+        self, catalogue: VelociraptorCatalogue, global_mask_tag: Union[None, str]
+    ):
         """
         Links a catalogue with this object so that the plots
         can actually be created.
@@ -1086,6 +1154,10 @@ class AutoPlotter(object):
 
         self.catalogue = catalogue
 
+        if global_mask_tag is not None:
+            self.global_mask = reduce(getattr, global_mask_tag.split("."), catalogue)
+        else:
+            self.global_mask = True
         return
 
     def create_plots(
@@ -1105,6 +1177,7 @@ class AutoPlotter(object):
 
         for plot in self.plots:
             try:
+                plot.global_mask = self.global_mask
                 plot.make_plot(
                     catalogue=self.catalogue,
                     directory=directory,
