@@ -238,7 +238,7 @@ def registration_angular_momentum(
         raise RegistrationDoesNotMatchError
 
     # All are angular momenta, so have same units.
-    unit = unit_system.length * unit_system.velocity
+    unit = unit_system.mass * unit_system.length * unit_system.velocity
 
     # Capture group 1: axis (x, y, z)
     # Capture group 2: radius within this was calculated, e.g. 200crit
@@ -307,6 +307,8 @@ def registration_masses(
         full_name = "$M_{\\rm FOF}$"
     elif field_path == "Mass_tot":
         full_name = r"$M$"
+    elif field_path == "Mass_interloper":
+        full_name = "$M_{\\rm BG}$"
 
     # General regex matching case.
 
@@ -321,7 +323,6 @@ def registration_masses(
     )
     regex = cached_regex(match_string)
     match = regex.match(field_path)
-
     if match and not full_name:
         mass = match.group(1)
         radius = match.group(2)
@@ -983,14 +984,14 @@ def registration_gas_hydrogen_species_masses(
     unit = unit_system.mass
 
     # Capture aperture size
-    match_string = "Aperture_([a-zA-Z]*)_aperture_total_gas_([0-9]*)_kpc"
+    match_string = "Aperture_([a-zA-Z]*)_(index_0_)?aperture_total_gas_([0-9]*)_kpc"
     regex = cached_regex(match_string)
 
     match = regex.match(field_path)
 
     if match:
         long_species = match.group(1)
-        aperture_size = match.group(2)
+        aperture_size = match.group(3)
 
         try:
             short_species = {
@@ -1022,44 +1023,147 @@ def registration_gas_hydrogen_species_masses(
         raise RegistrationDoesNotMatchError
 
 
-def registration_gas_element_ratios_times_masses(
+def registration_cold_dense_gas_properties(
     field_path: str, unit_system: VelociraptorUnits
 ) -> (unyt.Unit, str, str):
     """
-    Registers the Fe/H times mass and O/H times mass within apertures
+    Registers the mass of cold (T < 10^4.5 K), dense (nH > 0.1 cm^-3) gas in apertures.
+    """
+    unit = unit_system.mass
+
+    # Capture aperture size
+    match_string = "Aperture_ColdDense([a-zA-Z]*)Masses_aperture_total_gas_([0-9]*)_kpc"
+    regex = cached_regex(match_string)
+    match = regex.match(field_path)
+
+    if match:
+        quantity_key = match.group(1)
+        aperture_size = match.group(2)
+
+        try:
+            long_quantity = {"DiffuseMetal": "Diffuse Metal", "Gas": "Gas"}[
+                quantity_key
+            ]
+            short_quantity = {"DiffuseMetal": "diffuse_metal", "Gas": "gas"}[
+                quantity_key
+            ]
+        except KeyError:
+            raise RegistrationDoesNotMatchError
+        full_name = (
+            f"{long_quantity} Masses in Cold, Dense ($T < 10^{{4.5}}\;{{\rm K}}$, "
+            f"$n_{{\\rm H}}$ > 0.1 \\; {{\rm cm^{{-3}}}}$) Gas ({aperture_size} kpc)"
+        )
+        snake_case = f"cold_dense_{short_quantity}_mass_{aperture_size}_kpc"
+        return unit, full_name, snake_case
+    else:
+        raise RegistrationDoesNotMatchError
+
+
+def registration_log_element_ratios_times_masses(
+    field_path: str, unit_system: VelociraptorUnits
+) -> (unyt.Unit, str, str):
+    """
+    Registers the log10(Fe/H) times mass and log10(O/H) times mass within apertures for two particle floor values
     """
 
     unit = unit_system.mass
 
     # Capture aperture size
-    match_string = "Aperture_([a-zA-Z]*)_aperture_total_gas_([0-9]*)_kpc"
+    match_string = (
+        "Aperture_([a-zA-Z]*)Masses(Lo|Hi)Floor_aperture_total_([a-zA-Z]*)_([0-9]*)_kpc"
+    )
     regex = cached_regex(match_string)
 
     match = regex.match(field_path)
 
     if match:
         long_species = match.group(1)
-        aperture_size = match.group(2)
+        floor_type = match.group(2)
+        part_type = match.group(3)
+        aperture_size = match.group(4)
+        try:
+            short_species = {
+                "LogOxygenOverHydrogen": "O_over_H",
+                "LogIronOverHydrogen": "Fe_over_H",
+                "LogSNIaIronOverHydrogen": "SNIaFe_over_H",
+                "LogOxygenOverHydrogenAtomic": "O_over_H_atomic",
+                "LogOxygenOverHydrogenMolecular": "O_over_H_molecular",
+            }[long_species]
+            element_name = {
+                "LogOxygenOverHydrogen": "Oxygen",
+                "LogIronOverHydrogen": "Iron",
+                "LogSNIaIronOverHydrogen": "SNIa Iron",
+                "LogOxygenOverHydrogenAtomic": "Atomic-phase Oxygen",
+                "LogOxygenOverHydrogenMolecular": "Molecular-phase Oxygen",
+            }[long_species]
+            fraction_name = {
+                "LogOxygenOverHydrogen": "O/H",
+                "LogIronOverHydrogen": "Fe/H",
+                "LogSNIaIronOverHydrogen": "Fe_SNIa/H",
+                "LogOxygenOverHydrogenAtomic": "O/H",
+                "LogOxygenOverHydrogenMolecular": "O/H",
+            }[long_species]
+
+            short_floortype = {"Lo": "lowfloor", "Hi": "highfloor"}[floor_type]
+            floor_value = {"Lo": "-4", "Hi": "-3"}[floor_type]
+        except KeyError:
+            raise RegistrationDoesNotMatchError
+
+        full_name = (
+            f"Log10 {element_name} Abundance Weighted {part_type.capitalize()} Mass ({fraction_name}) "
+            f"$\times M_{{\\rm gas}}$, from particle values floored at [{fraction_name}]$\\gtreq {floor_value}$ "
+            f"({aperture_size} kpc)"
+        )
+        snake_case = f"log_{short_species}_times_{part_type}_mass_{short_floortype}_{aperture_size}_kpc"
+        return unit, full_name, snake_case
+    else:
+        raise RegistrationDoesNotMatchError
+
+
+def registration_lin_element_ratios_times_masses(
+    field_path: str, unit_system: VelociraptorUnits
+) -> (unyt.Unit, str, str):
+    """
+    Registers the Fe/H times mass and O/H times mass within apertures for two particle floor values
+    """
+
+    unit = unit_system.mass
+
+    # Capture aperture size
+    match_string = "Aperture_([a-zA-Z]*)Masses_aperture_total_([a-zA-Z]*)_([0-9]*)_kpc"
+    regex = cached_regex(match_string)
+
+    match = regex.match(field_path)
+
+    if match:
+        long_species = match.group(1)
+        part_type = match.group(2)
+        aperture_size = match.group(3)
 
         try:
             short_species = {
-                "OxygenOverHydrogenMasses": "O_over_H",
-                "IronOverHydrogenMasses": "Fe_over_H",
+                "TotalOxygenOverHydrogen": "O_over_H_total",
+                "OxygenOverHydrogen": "O_over_H",
+                "IronOverHydrogen": "Fe_over_H",
             }[long_species]
             element_name = {
-                "OxygenOverHydrogenMasses": "Oxygen",
-                "IronOverHydrogenMasses": "Iron",
+                "TotalOxygenOverHydrogen": "Oxygen",
+                "OxygenOverHydrogen": "Oxygen",
+                "IronOverHydrogen": "Iron",
             }[long_species]
             fraction_name = {
-                "OxygenOverHydrogenMasses": "(O/H)",
-                "IronOverHydrogenMasses": "(Fe/H)",
+                "TotalOxygenOverHydrogen": "O/H",
+                "OxygenOverHydrogen": "O/H",
+                "IronOverHydrogen": "Fe/H",
             }[long_species]
         except KeyError:
             raise RegistrationDoesNotMatchError
 
-        full_name = f"{element_name} Abundance Weighted Gas Mass {fraction_name}$\times M_{{\\rm gas}}$ ({aperture_size} kpc)"
-        snake_case = f"{short_species}_times_gas_mass_{aperture_size}_kpc"
-
+        full_name = (
+            f"Linear {element_name} Abundance Weighted {part_type.capitalize()} Mass ({fraction_name}) "
+            f"$\times M_{{\\rm gas}}$ ({aperture_size} kpc)"
+        )
+        snake_case = f"lin_{short_species}_times_{part_type}_mass_{aperture_size}_kpc"
         return unit, full_name, snake_case
     else:
         raise RegistrationDoesNotMatchError
@@ -1088,11 +1192,44 @@ def registration_dust_masses(
             short_species = {
                 "GraphiteMasses": "graphite",
                 "SilicatesMasses": "silicates",
+                "SmallGrainMasses": "small_grain",
+                "LargeGrainMasses": "large_grain",
+                "GraphiteMassesAtomic": "atomic_graphite",
+                "SilicatesMassesAtomic": "atomic_silicates",
+                "SmallGrainMassesAtomic": "atomic_small_grain",
+                "LargeGrainMassesAtomic": "atomic_large_grain",
+                "GraphiteMassesMolecular": "molecular_graphite",
+                "SilicatesMassesMolecular": "molecular_silicates",
+                "SmallGrainMassesMolecular": "molecular_small_grain",
+                "LargeGrainMassesMolecular": "molecular_large_grain",
+                "GraphiteMassesColdDense": "cold_dense_graphite",
+                "SilicatesMassesColdDense": "cold_dense_silicates",
+                "SmallGrainMassesColdDense": "cold_dense_small_grain",
+                "LargeGrainMassesColdDense": "cold_dense_large_grain",
             }[long_species]
+            pretty_name = {
+                "GraphiteMasses": "Graphite Dust Mass",
+                "SilicatesMasses": "Silicate Dust Mass",
+                "SmallGrainMasses": "small_grain",
+                "LargeGrainMasses": "large_grain",
+                "GraphiteMassesAtomic": "Graphite Dust Mass in Atomic Gas",
+                "SilicatesMassesAtomic": "Silicate Dust Mass in Atomic Gas",
+                "SmallGrainMassesAtomic": "atomic_small_grain",
+                "LargeGrainMassesAtomic": "atomic_large_grain",
+                "GraphiteMassesMolecular": "Graphite Dust Mass in Molecular Gas",
+                "SilicatesMassesMolecular": "Silicate Dust Mass in Molecular Gas",
+                "SmallGrainMassesMolecular": "molecular_small_grain",
+                "LargeGrainMassesMolecular": "molecular_large_grain",
+                "GraphiteMassesColdDense": "Graphite Dust Mass in Cold-Dense Gas",
+                "SilicatesMassesColdDense": "Silicate Dust Mass in Cold-Dense Gas",
+                "SmallGrainMassesColdDense": "cold_dense_small_grain",
+                "LargeGrainMassesColdDense": "cold_dense_large_grain",
+            }[long_species]
+
         except KeyError:
             raise RegistrationDoesNotMatchError
 
-        full_name = f"{short_species} Dust Mass ({aperture_size} kpc)"
+        full_name = f"{pretty_name} ({aperture_size} kpc)"
         snake_case = f"{short_species}_mass_{aperture_size}_kpc"
 
         return unit, full_name, snake_case
@@ -1111,14 +1248,16 @@ def registration_stellar_luminosities(
 
     # Capture aperture size
     match_string = (
-        "Aperture_Luminosities_([a-zA-Z])_index_0_aperture_total_star_([0-9]*)_kpc"
+        "Aperture_Luminosities_index_([0-9]?)_aperture_total_star_([0-9]*)_kpc"
     )
     regex = cached_regex(match_string)
 
     match = regex.match(field_path)
 
     if match:
-        band = match.group(1)
+
+        bands = ["u", "g", "r", "i", "z", "Y", "J", "H", "K"]
+        band = bands[int(match.group(1))]
         aperture_size = match.group(2)
 
         full_name = f"{band}-Band Luminosity ({aperture_size} kpc)"
@@ -1346,6 +1485,44 @@ def registration_spherical_overdensities(
         raise RegistrationDoesNotMatchError
 
 
+def registration_element_masses_in_stars(
+    field_path: str, unit_system: VelociraptorUnits
+) -> (unyt.Unit, str, str):
+    """
+    Registers element masses contained in stars
+    """
+
+    unit = unit_system.mass
+
+    # Capture aperture size
+    match_string = "Aperture_([a-zA-Z]*)Masses_aperture_total_star_([0-9]*)_kpc"
+    regex = cached_regex(match_string)
+    match = regex.match(field_path)
+
+    if match:
+        element = match.group(1)
+        aperture_size = match.group(2)
+
+        try:
+            field = {
+                "Oxygen": "Total Oxygen mass in stars",
+                "Magnesium": "Total Magnesium mass in stars",
+                "Iron": "Total Iron mass in stars",
+            }[element]
+        except KeyError:
+            raise RegistrationDoesNotMatchError
+
+        full_name = f"{field} computed in apertures of size ({aperture_size} kpc)"
+
+        snake_case = f"{element.lower()}_mass_{aperture_size}_kpc"
+
+        return unit, full_name, snake_case
+    else:
+        raise RegistrationDoesNotMatchError
+
+    return
+
+
 # TODO
 # lambda_B
 # q
@@ -1401,8 +1578,11 @@ global_registration_functions = {
         "gas_diffuse_element_masses",
         "dust_masses_from_table",
         "dust_masses",
-        "gas_element_ratios_times_masses",
         "stellar_luminosities",
+        "cold_dense_gas_properties",
+        "log_element_ratios_times_masses",
+        "lin_element_ratios_times_masses",
+        "element_masses_in_stars",
         "fail_all",
     ]
 }
